@@ -28,7 +28,7 @@ func main() {
 		&app.NicsToBridgeCommand,
 		&app.BridgesToNicCommand,
 		&app.ReadinessProbeCommand,
-		&app.OvnDBExporterCommand,
+		&app.OvsExporterCommand,
 	}
 
 	c.Before = func(ctx *cli.Context) error {
