@@ -37,36 +37,12 @@ func (ovn *Controller) getLoadBalancer(protocol kapi.Protocol) (string, error) {
 		return "", err
 	}
 	if out == "" {
-		return "", fmt.Errorf("no load-balancer found in the database")
+		return "", fmt.Errorf("no load balancer found in the database")
 	}
 	ovn.loadbalancerClusterCache[protocol] = out
 	return out, nil
 }
 
-// getDefaultGatewayLoadBalancer returns the load balancer for the node with the lowest gateway IP.
-// This is used in the implementation of ExternalIPs
-func (ovn *Controller) getDefaultGatewayLoadBalancer(protocol kapi.Protocol) string {
-	if outStr, ok := ovn.loadbalancerGWCache[protocol]; ok {
-		return outStr
-	}
-
-	gw, _, err := getDefaultGatewayRouterIP()
-	if err != nil {
-		klog.Errorf(err.Error())
-		return ""
-	}
-
-	externalIDKey := string(protocol) + "_lb_gateway_router"
-	lb, _, _ := util.RunOVNNbctl("--data=bare",
-		"--no-heading", "--columns=_uuid", "find", "load_balancer",
-		"external_ids:"+externalIDKey+"="+gw)
-	if len(lb) != 0 {
-		ovn.loadbalancerGWCache[protocol] = lb
-		ovn.defGatewayRouter = gw
-	}
-	return lb
-}
-
 // getLoadBalancerVIPs returns a map whose keys are VIPs (IP:port) on loadBalancer
 func (ovn *Controller) getLoadBalancerVIPs(loadBalancer string) (map[string]interface{}, error) {
 	outStr, _, err := util.RunOVNNbctl("--data=bare", "--no-heading",
@@ -75,7 +51,7 @@ func (ovn *Controller) getLoadBalancerVIPs(loadBalancer string) (map[string]inte
 		return nil, err
 	}
 	if outStr == "" {
-		return nil, nil
+		return nil, fmt.Errorf("load balancer vips in OVN DB is an empty string")
 	}
 	// sample outStr:
 	// - {"192.168.0.1:80"="10.1.1.1:80,10.2.2.2:80"}
@@ -91,19 +67,19 @@ func (ovn *Controller) getLoadBalancerVIPs(loadBalancer string) (map[string]inte
 }
 
 // deleteLoadBalancerVIP removes the VIP as well as any reject ACLs associated to the LB
-func (ovn *Controller) deleteLoadBalancerVIP(loadBalancer, vip string) {
+func (ovn *Controller) deleteLoadBalancerVIP(loadBalancer, vip string) error {
 	vipQuotes := fmt.Sprintf("\"%s\"", vip)
 	stdout, stderr, err := util.RunOVNNbctl("--if-exists", "remove", "load_balancer", loadBalancer, "vips", vipQuotes)
 	if err != nil {
-		klog.Errorf("Error in deleting load balancer vip %s for %s"+
+		// if we hit an error and fail to remove load balancer, we skip removing the rejectACL
+		return fmt.Errorf("error in deleting load balancer vip %s for %s"+
 			"stdout: %q, stderr: %q, error: %v",
 			vip, loadBalancer, stdout, stderr, err)
-		// if we hit an error and fail to remove load balancer, we skip removing the rejectACL
-		return
 	}
 	ovn.removeServiceEndpoints(loadBalancer, vip)
 	ovn.deleteLoadBalancerRejectACL(loadBalancer, vip)
 	ovn.removeServiceLB(loadBalancer, vip)
+	return nil
 }
 
 // configureLoadBalancer updates the VIP for sourceIP:sourcePort to point to targets (an
@@ -121,7 +97,7 @@ func (ovn *Controller) configureLoadBalancer(lb, sourceIP string, sourcePort int
 			"stdout: %q, stderr: %q, error: %v", lb, out, stderr, err)
 	}
 	ovn.setServiceEndpointsToLB(lb, vip, targets)
-	klog.V(5).Infof("lb entry set for %s, %s, %v", lb, lbTarget,
+	klog.V(5).Infof("LB entry set for %s, %s, %v", lb, lbTarget,
 		ovn.serviceLBMap[lb][vip])
 	return nil
 }
@@ -185,6 +161,33 @@ func (ovn *Controller) getLogicalSwitchesForLoadBalancer(lb string) ([]string, e
 	return nil, fmt.Errorf("router detected with load balancer that is not a GR")
 }
 
+// TODO: Add unittest for function.
+func (ovn *Controller) generateACLName(lb string, sourceIP string, sourcePort int32) string {
+	aclName := fmt.Sprintf("%s-%s:%d", lb, sourceIP, sourcePort)
+	// ACL names are limited to 63 characters
+	if len(aclName) > 63 {
+		var ipPortLen int
+		srcPortStr := fmt.Sprintf("%d", sourcePort)
+		// Add the length of the IP (max 15 with periods, max 39 with colons),
+		// plus length of sourcePort (max 5 char),
+		// plus 1 for additional ':' to separate,
+		// plus 1 for '-' between lb and IP.
+		// With full IPv6 address and 5 char port, max ipPortLen is 62
+		// With full IPv4 address and 5 char port, max ipPortLen is 24.
+		ipPortLen = len(sourceIP) + len(srcPortStr) + 1 + 1
+		lbTrim := 63 - ipPortLen
+		// Shorten the Load Balancer name to allow full IP:port
+		tmpLb := lb[:lbTrim]
+		klog.Infof("Limiting ACL Name from %s to %s-%s:%d to keep under 63 characters", aclName, tmpLb, sourceIP, sourcePort)
+		aclName = fmt.Sprintf("%s-%s:%d", tmpLb, sourceIP, sourcePort)
+	}
+	return aclName
+}
+
+func (ovn *Controller) generateACLNameForOVNCommand(lb string, sourceIP string, sourcePort int32) string {
+	return strings.ReplaceAll(ovn.generateACLName(lb, sourceIP, sourcePort), ":", "\\:")
+}
+
 func (ovn *Controller) createLoadBalancerRejectACL(lb string, sourceIP string, sourcePort int32, proto kapi.Protocol) (string, error) {
 	ovn.serviceLBLock.Lock()
 	defer ovn.serviceLBLock.Unlock()
@@ -211,11 +214,11 @@ func (ovn *Controller) createLoadBalancerRejectACL(lb string, sourceIP string, s
 	}
 	vip := util.JoinHostPortInt32(sourceIP, sourcePort)
 	// NOTE: doesn't use vip, to avoid having brackets in the name with IPv6
-	aclName := fmt.Sprintf("%s-%s:%d", lb, sourceIP, sourcePort)
+	aclName := ovn.generateACLNameForOVNCommand(lb, sourceIP, sourcePort)
 	// If ovn-k8s was restarted, we lost the cache, and an ACL may already exist in OVN. In that case we need to check
 	// using ACL name
 	aclUUID, stderr, err := util.RunOVNNbctl("--data=bare", "--no-heading", "--columns=_uuid", "find", "acl",
-		fmt.Sprintf("name=%s", strings.ReplaceAll(aclName, ":", "\\:")))
+		fmt.Sprintf("name=%s", aclName))
 	if err != nil {
 		klog.Errorf("Error while querying ACLs by name: %s, %v", stderr, err)
 	} else if len(aclUUID) > 0 {
@@ -239,7 +242,7 @@ func (ovn *Controller) createLoadBalancerRejectACL(lb string, sourceIP string, s
 		strings.ToLower(string(proto)), strings.ToLower(string(proto)), sourcePort)
 
 	cmd := []string{"--id=@acl", "create", "acl", "direction=from-lport", "priority=1000", aclMatch, "action=reject",
-		fmt.Sprintf("name=%s", strings.ReplaceAll(aclName, ":", "\\:"))}
+		fmt.Sprintf("name=%s", aclName)}
 	for _, ls := range switches {
 		cmd = append(cmd, "--", "add", "logical_switch", ls, "acls", "@acl")
 	}
@@ -257,12 +260,43 @@ func (ovn *Controller) createLoadBalancerRejectACL(lb string, sourceIP string, s
 }
 
 func (ovn *Controller) deleteLoadBalancerRejectACL(lb, vip string) {
-	acl, _ := ovn.getServiceLBInfo(lb, vip)
-	if acl == "" {
+	aclUUID, hasEndpoints := ovn.getServiceLBInfo(lb, vip)
+	if aclUUID == "" && !hasEndpoints {
+		// If no ACL and does not have endpoints, we can assume there is no valid entry in the cache here as
+		// this is an illegal state.
+		// Determine and remove ACL by name.
+		ip, port, err := util.SplitHostPortInt32(vip)
+		if err != nil {
+			klog.Errorf("Unable to parse vip for Reject ACL deletion: %v", err)
+			return
+		}
+		ovn.removeStaleRejectACL(lb, ip, port)
+	} else if aclUUID == "" {
+		// Must have endpoints and no reject ACL to remove
 		klog.V(5).Infof("No reject ACL found to remove for load balancer: %s, vip: %s", lb, vip)
 		return
+	} else {
+		ovn.removeACLFromNodeSwitches(lb, aclUUID)
+	}
+	ovn.removeServiceACL(lb, vip)
+}
+
+func (ovn *Controller) removeStaleRejectACL(lb, ip string, port int32) {
+	aclName := ovn.generateACLNameForOVNCommand(lb, ip, port)
+	aclUUID, stderr, err := util.RunOVNNbctl("--data=bare", "--no-heading", "--columns=_uuid", "find", "acl",
+		fmt.Sprintf("name=%s", aclName))
+	if err != nil {
+		klog.Errorf("Error while querying ACLs by name: %s, %v", stderr, err)
+		return
+	} else if len(aclUUID) == 0 {
+		klog.Infof("Reject ACL not found to remove for name: %s", aclName)
+		return
 	}
 
+	ovn.removeACLFromNodeSwitches(lb, aclUUID)
+}
+
+func (ovn *Controller) removeACLFromNodeSwitches(lb, aclUUID string) {
 	switches, err := ovn.getLogicalSwitchesForLoadBalancer(lb)
 	if err != nil {
 		klog.Errorf("Could not retrieve logical switches associated with load balancer %s", lb)
@@ -271,17 +305,15 @@ func (ovn *Controller) deleteLoadBalancerRejectACL(lb, vip string) {
 
 	args := []string{}
 	for _, ls := range switches {
-		args = append(args, "--", "--if-exists", "remove", "logical_switch", ls, "acl", acl)
+		args = append(args, "--", "--if-exists", "remove", "logical_switch", ls, "acl", aclUUID)
 	}
 
 	if len(args) > 0 {
 		_, _, err = util.RunOVNNbctl(args...)
 		if err != nil {
-			klog.Errorf("Error while removing ACL: %s, from switches, error: %v", acl, err)
+			klog.Errorf("Error while removing ACL: %s, from switches, error: %v", aclUUID, err)
 		} else {
-			klog.V(5).Infof("ACL: %s, removed from switches: %s", acl, switches)
+			klog.Infof("ACL: %s, removed from switches: %s", aclUUID, switches)
 		}
 	}
-
-	ovn.removeServiceACL(lb, vip)
 }
